using System.Reflection;
using Sample.Validation;

namespace Sample.Domain
{
    public class Person
    {
        private string firstName;
        private string lastName;
        private string address;
        private int age;
        private Country countryOfResidence;
        private Gender gender;
        private readonly IBusinessRuleSet<Person> ruleSet;

        public Person(string firstName, string lastName, string address, int age, Country countryOfResidence, Gender gender):this(firstName,lastName,address,age,countryOfResidence,gender,Rules.Default)
        {
            
        }

        public Person(string firstName, string lastName, string address, int age, Country countryOfResidence, Gender gender, IBusinessRuleSet<Person> ruleSet)
        {
            this.firstName = firstName;
            this.lastName = lastName;
            this.address = address;
            this.age = age;
            this.countryOfResidence = countryOfResidence;
            this.gender = gender;
            this.ruleSet = ruleSet;
        }

        public IBusinessRuleSet<Person> Validate()
        {
            return ruleSet.BrokenBy(this);
        }


        public string FirstName
        {
            get { return firstName; }
        }

        public string LastName
        {
            get { return lastName; }
        }

        public string Address
        {
            get { return address; }
        }

        public int Age
        {
            get { return age; }
        }

        public Country CountryOfResidence
        {
            get { return countryOfResidence; }
        }

        public Gender Gender
        {
            get { return gender; }
        }
        
        public sealed class Rules
        {
            private Rules()
            {
                
            }
            
            public static IBusinessRule<Person> FirstName
            {
                get
                {
                    return new BusinessRule<Person>(MethodInfo.GetCurrentMethod().Name,"First name is required",delegate(Person person)
                                                        {
                                                            return new NonEmptyStringSpecification().IsSatisfiedBy(person.FirstName);
                                                        });
                }
            }

            public static IBusinessRule<Person> LastName
            {
                get
                {
                    return new BusinessRule<Person>(MethodInfo.GetCurrentMethod().Name,"Last name is required",delegate(Person person)
                                                        {
                                                            return new NonEmptyStringSpecification().IsSatisfiedBy(person.LastName);
                                                        });
                }
            }

            public static IBusinessRule<Person> Address
            {
                get
                {
                    return new BusinessRule<Person>(MethodInfo.GetCurrentMethod().Name,"Address is required",delegate(Person person)
                                                        {
                                                            return new NonEmptyStringSpecification().IsSatisfiedBy(person.Address);
                                                        });
                }
            }

            public static IBusinessRule<Person> Age
            {
                get
                {
                    return new BusinessRule<Person>(MethodInfo.GetCurrentMethod().Name,"Must be older than 0",delegate(Person person)
                                                        {
                                                            return person.Age > 0;
                                                        });
                }
            }

            public static IBusinessRule<Person> Country
            {
                get
                {
                    return new BusinessRule<Person>(MethodInfo.GetCurrentMethod().Name,"Country is required",delegate(Person person)
                                                        {
                                                            return person.CountryOfResidence != null;
                                                        });
                }
            }


            public static IBusinessRule<Person> Gender
            {
                get
                {
                    return new BusinessRule<Person>(MethodInfo.GetCurrentMethod().Name,"Gender is required.",delegate(Person person)
                                                        {
                                                            return person.Gender != null;
                                                        });
                }
            }

            
            public static IBusinessRuleSet<Person> Default
            {
                get
                {
                    return new BusinessRuleSet<Person>(FirstName, LastName, Age, Address, Country, Gender);
                }
            }
        }
    }
}